@@ -50,7 +50,7 @@ slinteract <- function(sl, facts, mf) { # sl and facts are  logical
 
 getfl <- function(mf) {
 
-  facts <- vapply(unattrib(mf), is.factor, TRUE)
+  facts <- .Call(C_vtypes, mf, 2L) # vapply(unattrib(mf), is.factor, TRUE)
 
   # Any factors
   if(any(facts)) {
@@ -74,7 +74,7 @@ getfl <- function(mf) {
 
       fctdat <- NULL # best way to do this ?? or as before with pre-allocation ??
       lsf <- length(singlefct)
-      lff <- sum(fctfct)
+      lff <- bsum(fctfct)
       if(lsf) fctdat <- mf[singlefct] # unattrib() -> wrap around at the end... Nah, better with names...
       if(lff) fctdat <- c(fctdat, lapply(intterms[fctfct], finteract, TRUE, mf))
 
@@ -94,7 +94,7 @@ getfl <- function(mf) {
           # Check for duplicate factors in interactions (largely independent of the other stuff)
           dupchk <- factors[, -im, drop = FALSE] > 0L # same as intslopes...
           if(any(dupfct <- rowSums(dupchk) > 1)) { # Check for factors with multiple slopes...
-            if(sum(dupfct) > 1L) stop("Cannot currently support multiple factors with multiple slopes...")
+            if(bsum(dupfct) > 1L) stop("Cannot currently support multiple factors with multiple slopes...")
             dupfct <- which(dupchk[dupfct, ]) # This accounts for im
             fctdat <- c(fctdat, lapply(c(intslope[-im][dupfct[1L]], intslope[-im][-dupfct]), finteract, facts, mf))
           } else
@@ -114,7 +114,7 @@ getfl <- function(mf) {
             }
             if(any(dupfct)) { # reordering if dupfct... putting it in front..
               slopes[-lim] <- c(slopes[-lim][dupfct], slopes[-lim][-dupfct])
-              othmc <- c(sum(othmc[dupfct]), othmc[-dupfct])
+              othmc <- c(bsum(othmc[dupfct]), othmc[-dupfct])
             }
             slflag <- c(slflag, othmc)
           }
@@ -124,7 +124,7 @@ getfl <- function(mf) {
         } else { # No double factor interactions with slopes.. Only simple slopes interactions.. (what about dupfact of two different double interactions with slope, but no factfact?)
           dupchk <- factors > 0L # same as intslopes...
           if(any(dupfct <- rowSums(dupchk) > 1)) { # Check for factors with multiple slopes...
-            if(sum(dupfct) > 1L) stop("Cannot currently support multiple factors with multiple slopes...")
+            if(bsum(dupfct) > 1L) stop("Cannot currently support multiple factors with multiple slopes...")
             dupfct <- which(dupchk[dupfct, ])
             fctdat <- c(fctdat, lapply(c(intslope[dupfct[1L]], intslope[-dupfct]), finteract, facts, mf))
           } else fctdat <- c(fctdat, lapply(intslope, finteract, facts, mf))
@@ -136,7 +136,7 @@ getfl <- function(mf) {
           }
           if(any(dupfct)) { # reordering if dupfct... putting it in front..
             slopes <- c(slopes[dupfct], slopes[-dupfct])
-            lsl <- c(sum(lsl[dupfct]), lsl[-dupfct])
+            lsl <- c(bsum(lsl[dupfct]), lsl[-dupfct])
           }
           slflag <- c(slflag, integer(lff), lsl)
         }
@@ -278,7 +278,7 @@ fhdwithin.default <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.me
   if(na.rm) {
     cc <- complete.cases(x, fl, w) # gives error if lengths don't match, otherwise demeanlist and qr.resid give errors !!
     if(!all(cc)) {
-      ax[["na.rm"]] <- which(!cc)
+      ax[["na.rm"]] <- whichv(cc, FALSE)
       cc <- which(cc)
       w <- w[cc] # Note this here !!
       if(!fill) {
@@ -289,7 +289,7 @@ fhdwithin.default <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.me
   }
 
   if(is.list(fl)) {
-    fc <- vapply(unattrib(fl), is.factor, TRUE)
+    fc <- .Call(C_vtypes, fl, 2L) # vapply(unattrib(fl), is.factor, TRUE)
     fcl <- any(fc)
     # if(!fcl && !missing(...)) unused_arg_action(match.call(), ...)
     nallfc <- fcl && !all(fc)
@@ -331,7 +331,7 @@ fhdwithin.pseries <- function(x, effect = seq_col(attr(x, "index")), w = NULL, n
   namix <- attr(ix, "names")
   effect <- cols2int(effect, ix, namix)
   g <- .subset(ix, effect)
-  if(na.rm && length(cc <- which(!is.na(x))) != length(x)) {
+  if(na.rm && length(cc <- whichv(x, NA, TRUE)) != length(x)) {
      g <- .Call(C_subsetDT, g, cc, seq_along(g), FALSE) # lapply(g, `[`, cc) -> slower !
     if(fill) {
       x[cc] <- demean(.subset(`names<-`(x, NULL), cc), g, w[cc], ...) # keeps attributes ?? -> Yes !!
@@ -357,7 +357,7 @@ fhdwithin.matrix <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.met
   if(na.rm) {
     cc <- complete.cases(x, fl, w) # gives error if lengths don't match, otherwise demeanlist and qr.resid give errors !!
     if(!all(cc)) {
-      ax[["na.rm"]] <- which(!cc)
+      ax[["na.rm"]] <- whichv(cc, FALSE)
       cc <- which(cc)
       w <- w[cc]
       if(!fill) {
@@ -369,7 +369,7 @@ fhdwithin.matrix <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.met
   }
 
   if(is.list(fl)) {
-    fc <- vapply(unattrib(fl), is.factor, TRUE)
+    fc <- .Call(C_vtypes, fl, 2L) # vapply(unattrib(fl), is.factor, TRUE)
     fcl <- any(fc)
     # if(!fcl && !missing(...)) unused_arg_action(match.call(), ...)
     nallfc <- fcl && !all(fc)
@@ -418,13 +418,13 @@ fhdwithin.pdata.frame <- function(x, effect = seq_col(attr(x, "index")), w = NUL
     ax <- attributes(x)
     attributes(x) <- NULL
     varwisecomp <- function(x, fl, w, ...) lapply(x, function(y) {
-      ycc <- which(!is.na(y))
+      ycc <- whichv(y, NA, TRUE)
       y[ycc] <- demean(.subset(y, ycc), subsetfl(fl, ycc), w[ycc], ...)
       return(y)
     })
     return(setAttributes(varwisecomp(x, g, w, ...), ax))
   } else if(na.rm && any(miss <- .Call(C_dt_na, x, seq_along(unclass(x))))) {
-    cc <- which(!miss)
+    cc <- whichv(miss, FALSE)
     gcc <- .Call(C_subsetDT, g, cc, seq_along(g), FALSE)
     Y <- demean(.Call(C_subsetDT, x, cc, seq_along(unclass(x)), FALSE), gcc, w[cc], ...)
     if(fill) {
@@ -452,7 +452,7 @@ fhdwithin.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, va
   if(na.rm) {
     cc <- if(variable.wise) complete.cases(fl, w) else complete.cases(x, fl, w) # gives error if lengths don't match, otherwise demeanlist and qr.resid give errors !!
     if(!all(cc)) {
-      ax[["na.rm"]] <- which(!cc)
+      ax[["na.rm"]] <- whichv(cc, FALSE)
       cc <- which(cc)
       w <- w[cc]
       if(!variable.wise) {
@@ -463,7 +463,7 @@ fhdwithin.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, va
   }
 
   if(is.list(fl)) { # fl is a list !!
-    fc <- vapply(unattrib(fl), is.factor, TRUE)
+    fc <- .Call(C_vtypes, fl, 2L) # vapply(unattrib(fl), is.factor, TRUE)
     fcl <- any(fc)
     # if(!fcl && !missing(...)) unused_arg_action(match.call(), ...)
     nallfc <- fcl && !all(fc)
@@ -492,8 +492,8 @@ fhdwithin.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, va
     if(na.rm) { # this means there were mising values in fl, which were already removed!
       return(setAttributes(lapply(unattrib(x), function(y) {
         y[-cc] <- NA # which is not faster !!
-        ycc <- !is.na(y)
-        YC <- which(ycc[cc])
+        ycc <- whichv(y, NA, TRUE)
+        YC <- whichv(y[cc], NA, TRUE)
         wc <- w[YC]
         y[ycc] <- if(nallfc) flmres(demean(y[ycc], subsetfl(fl, YC), wc, ...), xmat[YC, , drop = FALSE], wc, lm.method, ...) else if(fcl)
           demean(y[ycc], subsetfl(fl, YC), wc, ...) else flmres(y[ycc], xmat[YC, , drop = FALSE], wc, lm.method, ...)
@@ -501,7 +501,7 @@ fhdwithin.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, va
       }), ax))
     }
     return(setAttributes(lapply(unattrib(x), function(y) {
-      ycc <- which(!is.na(y))
+      ycc <- whichv(y, NA, TRUE)
       wc <- w[ycc]
       y[ycc] <- if(nallfc) flmres(demean(y[ycc], subsetfl(fl, ycc), wc, ...), xmat[ycc, , drop = FALSE], wc, lm.method, ...) else if(fcl)
         demean(y[ycc], subsetfl(fl, ycc), wc, ...) else flmres(y[ycc], xmat[ycc, , drop = FALSE], wc, lm.method, ...)
@@ -551,7 +551,7 @@ HDW.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
       if(missw <- length(w) && anyNA(w)) miss <- miss | is.na(w)
       if(missw || any(miss)) {
         ax[["na.rm"]] <- which(miss)
-        cc <- which(!miss)
+        cc <- whichv(miss, FALSE)
         w <- w[cc]
         if(!variable.wise) if(fill) nrx <- fnrow2(x) else ax[["row.names"]] <- ax[["row.names"]][cc] # best ??
       } else na.rm <- FALSE
@@ -567,8 +567,8 @@ HDW.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
       if(na.rm) {
         return(setAttributes(lapply(.subset(x, Xvars), function(y) {
           y[-cc] <- NA
-          ycc <- !is.na(y)
-          YC <- which(ycc[cc])
+          ycc <- whichv(y, NA, TRUE)
+          YC <- whichv(y[cc], NA, TRUE)
           wc <- w[YC]
           y[ycc] <- if(nallfc) flmres(demean(y[ycc], subsetfl(fl, YC), wc, ...), xmat[YC, , drop = FALSE], wc, lm.method, ...) else if(fcl)
             demean(y[ycc], subsetfl(fl, YC), wc, ...) else  flmres(y[ycc], xmat[YC, , drop = FALSE], wc, lm.method, ...)
@@ -576,7 +576,7 @@ HDW.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
         }), ax))
       }
       return(setAttributes(lapply(.subset(x, Xvars), function(y) {
-        ycc <- which(!is.na(y))
+        ycc <- whichv(y, NA, TRUE)
         wc <- w[ycc]
         y[ycc] <- if(nallfc) flmres(demean(y[ycc], subsetfl(fl, ycc), wc, ...), xmat[ycc, , drop = FALSE], wc, lm.method, ...) else if(fcl)
           demean(y[ycc], subsetfl(fl, ycc), wc, ...) else flmres(y[ycc], xmat[ycc, , drop = FALSE], wc, lm.method, ...)
@@ -620,7 +620,7 @@ fhdbetween.default <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.m
   if(na.rm) {
     cc <- complete.cases(x, fl, w) # gives error if lengths don't match, otherwise demeanlist and qr.resid give errors !!
     if(!all(cc)) {
-      ax[["na.rm"]] <- which(!cc)
+      ax[["na.rm"]] <- whichv(cc, FALSE)
       cc <- which(cc)
       w <- w[cc] # Note this here !!
       if(!fill) {
@@ -631,7 +631,7 @@ fhdbetween.default <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.m
   }
 
   if(is.list(fl)) {
-    fc <- vapply(unattrib(fl), is.factor, TRUE)
+    fc <- .Call(C_vtypes, fl, 2L) # vapply(unattrib(fl), is.factor, TRUE)
     fcl <- any(fc)
     # if(!fcl && !missing(...)) unused_arg_action(match.call(), ...)
     nallfc <- fcl && !all(fc)
@@ -682,7 +682,7 @@ fhdbetween.matrix <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.me
   if(na.rm) {
     cc <- complete.cases(x, fl, w) # gives error if lengths don't match, otherwise demeanlist and qr.resid give errors !!
     if(!all(cc)) {
-      ax[["na.rm"]] <- which(!cc)
+      ax[["na.rm"]] <- whichv(cc, FALSE)
       cc <- which(cc)
       w <- w[cc]
       if(!fill) {
@@ -694,7 +694,7 @@ fhdbetween.matrix <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.me
   }
 
   if(is.list(fl)) {
-    fc <- vapply(unattrib(fl), is.factor, TRUE)
+    fc <- .Call(C_vtypes, fl, 2L) # vapply(unattrib(fl), is.factor, TRUE)
     fcl <- any(fc)
     # if(!fcl && !missing(...)) unused_arg_action(match.call(), ...)
     nallfc <- fcl && !all(fc)
@@ -746,7 +746,7 @@ fhdbetween.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, v
   if(na.rm) {
     cc <- if(variable.wise) complete.cases(fl, w) else complete.cases(x, fl, w) # gives error if lengths don't match, otherwise demeanlist and qr.resid give errors !!
     if(!all(cc)) {
-      ax[["na.rm"]] <- which(!cc)
+      ax[["na.rm"]] <- whichv(cc, FALSE)
       cc <- which(cc)
       w <- w[cc]
       if(!variable.wise) {
@@ -757,7 +757,7 @@ fhdbetween.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, v
   }
 
   if(is.list(fl)) { # fl is a list !!
-    fc <- vapply(unattrib(fl), is.factor, TRUE)
+    fc <- .Call(C_vtypes, fl, 2L) # vapply(unattrib(fl), is.factor, TRUE)
     fcl <- any(fc)
     # if(!fcl && !missing(...)) unused_arg_action(match.call(), ...)
     nallfc <- fcl && !all(fc)
@@ -786,8 +786,8 @@ fhdbetween.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, v
     if(na.rm) { # this means there were mising values in fl, which were already removed!
       return(setAttributes(lapply(unattrib(x), function(y) {
         y[-cc] <- NA # which is not faster !!
-        ycc <- !is.na(y)
-        YC <- which(ycc[cc])
+        ycc <- whichv(y, NA, TRUE)
+        YC <- whichv(y[cc], NA, TRUE)
         wc <- w[YC]
         yycc <- y[ycc]
         y[ycc] <- if(nallfc) yycc - flmres(demean(yycc, subsetfl(fl, YC), wc, ...), xmat[YC, , drop = FALSE], wc, lm.method, ...) else if(fcl)
@@ -796,7 +796,7 @@ fhdbetween.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, v
       }), ax))
     }
     return(setAttributes(lapply(unattrib(x), function(y) {
-      ycc <- which(!is.na(y))
+      ycc <- whichv(y, NA, TRUE)
       wc <- w[ycc]
       yycc <- y[ycc]
       y[ycc] <- if(nallfc) yycc - flmres(demean(yycc, subsetfl(fl, ycc), wc, ...), xmat[ycc, , drop = FALSE], wc, lm.method, ...) else if(fcl)
@@ -848,7 +848,7 @@ HDB.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
       if(missw <- length(w) && anyNA(w)) miss <- miss | is.na(w)
       if(missw || any(miss)) {
         ax[["na.rm"]] <- which(miss)
-        cc <- which(!miss)
+        cc <- whichv(miss, FALSE)
         w <- w[cc]
         if(!variable.wise) if(fill) nrx <- fnrow2(x) else ax[["row.names"]] <- ax[["row.names"]][cc] # best ??
       } else na.rm <- FALSE
@@ -866,8 +866,8 @@ HDB.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
       if(na.rm) { # this means there were mising values in fl, which were already removed!
         return(setAttributes(lapply(unattrib(x), function(y) {
           y[-cc] <- NA # which is not faster !!
-          ycc <- !is.na(y)
-          YC <- which(ycc[cc])
+          ycc <- whichv(y, NA, TRUE)
+          YC <- whichv(y[cc], NA, TRUE)
           wc <- w[YC]
           yycc <- y[ycc]
           y[ycc] <- if(nallfc) yycc - flmres(demean(yycc, subsetfl(fl, YC), wc, ...), xmat[YC, , drop = FALSE], wc, lm.method, ...) else if(fcl)
@@ -876,7 +876,7 @@ HDB.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
         }), ax))
       }
       return(setAttributes(lapply(unattrib(x), function(y) {
-        ycc <- which(!is.na(y))
+        ycc <- whichv(y, NA, TRUE)
         wc <- w[ycc]
         yycc <- y[ycc]
         y[ycc] <- if(nallfc) yycc - flmres(demean(yycc, subsetfl(fl, ycc), wc, ...), xmat[ycc, , drop = FALSE], wc, lm.method, ...) else if(fcl)
@@ -902,15 +902,40 @@ HDB.pdata.frame <- function(x, effect = seq_col(attr(x, "index")), w = NULL, col
   add_stub(fhdwithin.pdata.frame(if(is.null(cols)) x else colsubset(x, cols), effect, w, na.rm, fill, variable.wise, ..., means = TRUE), stub)
 
 
-fHDbetween <- fhdbetween
-fHDbetween.default <- function(x, ...) fhdbetween.default(x, ...)
-fHDbetween.matrix <- function(x, ...) fhdbetween.matrix(x, ...)
-fHDbetween.data.frame <- function(x, ...) fhdbetween.data.frame(x, ...)
+fHDbetween <- function(x, ...) {
+  message("Note that 'fHDbetween' was renamed to 'fhdbetween'. The S3 generic will not be removed anytime soon, but please use updated function names in new code, see help('collapse-renamed')")
+  UseMethod("fhdbetween")
+}
+fHDbetween.default <- function(x, ...) {
+  .Deprecated(msg = "This method belongs to a renamed function and will be removed end of 2022, see help('collapse-renamed')")
+  fhdbetween.default(x, ...)
+}
+fHDbetween.matrix <- function(x, ...) {
+  .Deprecated(msg = "This method belongs to a renamed function and will be removed end of 2022, see help('collapse-renamed')")
+  fhdbetween.matrix(x, ...)
+}
+fHDbetween.data.frame <- function(x, ...) {
+  .Deprecated(msg = "This method belongs to a renamed function and will be removed end of 2022, see help('collapse-renamed')")
+  fhdbetween.data.frame(x, ...)
+}
 
-fHDwithin <- fhdwithin
-fHDwithin.default <- function(x, ...) fhdwithin.default(x, ...)
-fHDwithin.matrix <- function(x, ...) fhdwithin.matrix(x, ...)
-fHDwithin.data.frame <- function(x, ...) fhdwithin.data.frame(x, ...)
+
+fHDwithin <- function(x, ...) {
+  message("Note that 'fHDwithin' was renamed to 'fhdwithin'. The S3 generic will not be removed anytime soon, but please use updated function names in new code, see help('collapse-renamed')")
+  UseMethod("fhdwithin")
+}
+fHDwithin.default <- function(x, ...) {
+  .Deprecated(msg = "This method belongs to a renamed function and will be removed end of 2022, see help('collapse-renamed')")
+  fhdwithin.default(x, ...)
+}
+fHDwithin.matrix <- function(x, ...) {
+  .Deprecated(msg = "This method belongs to a renamed function and will be removed end of 2022, see help('collapse-renamed')")
+  fhdwithin.matrix(x, ...)
+}
+fHDwithin.data.frame <- function(x, ...) {
+  .Deprecated(msg = "This method belongs to a renamed function and will be removed end of 2022, see help('collapse-renamed')")
+  fhdwithin.data.frame(x, ...)
+}
 
 
 #
